/*
    Module:
        KeyboardDisplay
    Author:
        Patrick Gillespie - http://patorjk.com/
    Code Licence:
        GPL - http://www.gnu.org/licenses/gpl.html
    notes:
        This is the definition of the KeyboardDisplay object/class
*/

KeyboardDisplay = (function () {

	// -------------------------------------------------------------------------
	// Private Static Varaibles and Functions
	// -------------------------------------------------------------------------

    // unquie ID for each Keyboard Display object that is created
    var KbId = 0;
    
    // Key Codes for the QWERTY keyboard

    var KeyCodeMap = {};
    
    KeyCodeMap["qwerty"] = {};
    KeyCodeMap["qwerty"].img = chrome.extension.getURL("images/qwerty_480.png");
    KeyCodeMap["qwerty"].layout = [
        [192,  49,   50,   51,   52,   53,   54,   55,   56,   57,   48,   189,  187,  8],
        [9,    81,   87,   69,   82,   84,   89,   85,   73,   79,   80,   219,  221,  220], 
        [20,   65,   83,   68,   70,   71,   72,   74,   75,   76,   186,  222,  13],
        [16,   90,   88,   67,   86,   66,   78,   77,   188,  190,  191,  16],
        [17,   91,   18,   32,   18,   91,   93,   17]
    ];

    KeyCodeMap["dvorak"] = {};
    KeyCodeMap["dvorak"].img = chrome.extension.getURL("images/dvorak_480.png");
    KeyCodeMap["dvorak"].layout = [
        [192,  49,   50,    51,     52,   53,   54,   55,   56,     57,   48,   219,  221,  8],
        [9,   222,   188,   190,    80,   89,   70,   71,   67,     82,   76,   191,  187,  220], 
        [20,   65,   79,    69,     85,   73,   68,   72,   84,     78,   83,   189,  13],
        [16,  186,   81,    74,     75,   88,   66,   77,   87,     86,   90,   16],
        [17,   91,   18,    32,     18,   91,   93,   17]
    ];
    
    KeyCodeMap["colemak"] = {};
    KeyCodeMap["colemak"].img = chrome.extension.getURL("images/colemak_480.png");
    KeyCodeMap["colemak"].layout = [
        [192,  49,   50,   51,   52,   53,   54,   55,   56,   57,   48,    189,  187,  8],
        [9,    81,   87,   70,   80,   71,   74,   76,   85,   89,   186,   219,  221,  220],
        [8,    65,   82,   83,   84,   68,   72,   78,   69,   73,   79,    222,  13],
        [16,   90,   88,   67,   86,   66,   75,   77,   188,  190,  191,  16],
        [17,   91,   18,   32,   18,   91,   93,   17]
    ];

    var KbKeyHeight = 32;
    var KbKeyWidths = [ 
        [32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 64],
        [48, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 48],
        [56, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 72],
        [72, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 88],
        [48, 32, 48, 192, 48, 32, 32, 48]
    ];
    
    var KeyCodeLabelMap = {
        "kc-192": "` ~",        "kc-49":"1 !", "kc-50":"2 @", "kc-51":"3 #", "kc-52":"4 $", "kc-53": "5 %", "kc-54":"6 ^", 
                                "kc-55":"7 &", "kc-56":"8 *", "kc-57":"9 (", "kc-48":"0 )", "kc-189":"- _", "kc-187":"= +", "kc-8":"BACKSPACE",
        "kc-9": "TAB",          "kc-81":"q Q", "kc-87":"w W", "kc-69":"e E", "kc-82":"r R", "kc-84":"t T",  "kc-89":"y Y", 
                                "kc-85":"u U", "kc-73":"i I", "kc-79":"o O", "kc-80":"p P", "kc-219":"[ {", "kc-221":"] }", "kc-220":"\\ |",
        "kc-20":"CAPS LOCK",    "kc-65":"a A", "kc-83":"s S", "kc-68":"d D", "kc-70":"f F", "kc-71":"g G",  "kc-72":"h H",   
                                "kc-74":"j J", "kc-75":"k K", "kc-76":"l L", "kc-186":"; :","kc-222":"' \"","kc-13":"ENTER",
        "kc-16":"SHIFT",        "kc-90":"z Z", "kc-88":"x X", "kc-67":"c C", "kc-86":"v V", "kc-66":"b B",  "kc-78":"n N",   
                                "kc-77":"m M", "kc-188":", <","kc-190":". >","kc-191":"/ ?","kc-16":"SHIFT",
        
        "kc-17":"CTRL",         "kc-91":"WIN KEY",   "kc-18":"ALT",   "kc-32":"SPACE",   "kc-18":"ALT",   "kc-91":"WIN KEY",   "kc-93":"MENU",   "kc-17":"CTRL"
    };

    
    
	// -------------------------------------------------------------------------
	// Constructor
	// -------------------------------------------------------------------------

    return function(config) {

    	// -------------------------------------------------------------------------
    	// Private Variables with default values
    	// -------------------------------------------------------------------------
    
        var that = this;
        var IsSetup = false;
        var MyId = KbId;
        var KbLayout = "qwerty";
        var KeyData = {};
        var VisualizationType = "speed";
        var ValidVisualizationType = {};
        ValidVisualizationType["speed"] = true;
        ValidVisualizationType["frequency"] = true;
        
        KbId++;
        
    	// -------------------------------------------------------------------------
    	// Private Methods
    	// -------------------------------------------------------------------------
        
        function isSetup(newValue) {
            if (!newValue) {
                return IsSetup;
            } else {
                IsSetup = newValue;
            }
        }
    
        function getId() {
            return MyId;
        }
    
    	// -------------------------------------------------------------------------
    	// Privileged Methods
    	// -------------------------------------------------------------------------
        
        this.setup = function(config) {
            if ( isSetup() ) {return;} // we've already been setup
            isSetup(true);
            
            // set style of keyboard display
            
            console.log("config.renderTo = " + config.renderTo);
            
            var d = document.getElementById(config.renderTo);
            console.log(d);
            d.style.width = "480px";
            d.style.height = "161px";
            d.style.padding = "0px";
            d.style.backgroundColor = "#fff";
            
            var dContent = document.createElement("div");
            dContent.id = "KeyboardDisplay_" + getId();
            dContent.style.width = "480px";
            dContent.style.height = "161px";
            dContent.style.position = "relative";
            d.appendChild(dContent);
            
            // create child divs for highlighing keyboard keys
            // these will exist under the keyboard image
            
            var xPos = 0;
            var yPos = 0;
            
            for (var ii = 0; ii < KbKeyWidths.length; ii++) {            
                xPos = 0;
                for (var jj = 0; jj < KbKeyWidths[ii].length; jj++) {
                    
                    var keyNode = document.createElement("div");
                    keyNode.id = "Kd_" + getId() + "_KeyHighlighter_" + ii + "_" + jj;
                    keyNode.style.padding = "0px";
                    keyNode.style.margin = "0px";
                    keyNode.style.position = "absolute";
                    keyNode.style.width = KbKeyWidths[ii][jj] + "px";
                    keyNode.style.height = KbKeyHeight + "px";
                    keyNode.style.left = xPos + "px";
                    keyNode.style.top = yPos + "px";
                    
                    dContent.appendChild(keyNode);
                    
                    xPos += KbKeyWidths[ii][jj];
                }
                yPos += KbKeyHeight;
            }
            
            // create background image and highlighting div
            
            var keyCover = document.createElement("div");
            keyCover.id = "Kd_" + getId() + "_KeyCover";
            keyCover.style.padding = "0px";
            keyCover.style.margin = "0px";
            keyCover.style.width = "480px";
            keyCover.style.height = "161px";
            keyCover.style.position = "absolute";
            keyCover.style.backgroundImage = "url('" + this.getKeyoutLayoutImg() + "')";
            dContent.appendChild(keyCover);
            
            var highlightDiv = document.createElement("div");
            highlightDiv.id = "Kd_" + getId() + "_KeyHighlighter";
            highlightDiv.style.position = "absolute";
            highlightDiv.style.display = "none";
            highlightDiv.style.padding = "0px";
            dContent.appendChild(highlightDiv);
            
            var mouseOverKeyLabel = null;
            if (config.mouseOverKeyLabel) {
                mouseOverKeyLabel = document.getElementById(config.mouseOverKeyLabel);
            }
            var mouseOverTimesPressedLabel = null;
            if (config.mouseOverTimesPressedLabel) {
                mouseOverTimesPressedLabel = document.getElementById(config.mouseOverTimesPressedLabel);
            }
            var mouseOverAverageSpeedLabel = null;
            if (config.mouseOverAverageSpeedLabel) {
                mouseOverAverageSpeedLabel = document.getElementById(config.mouseOverAverageSpeedLabel);
            }
            
            // create child divs for detecting mouse over/out events
            // for the keyboard keys
            // these will exist above the keyboard image
            
            var xPos = 0;
            var yPos = 0;
            
            for (var ii = 0; ii < KbKeyWidths.length; ii++) {            
                xPos = 0;
                for (var jj = 0; jj < KbKeyWidths[ii].length; jj++) {
                    
                    var keyNode = document.createElement("div");
                    keyNode.id = "Kd_" + getId() + "_Key_" + ii + "_" + jj;
                    keyNode.style.padding = "0px";
                    keyNode.style.margin = "0px";
                    keyNode.style.position = "absolute";
                    keyNode.style.width = KbKeyWidths[ii][jj] + "px";
                    keyNode.style.height = KbKeyHeight + "px";
                    keyNode.style.left = xPos + "px";
                    keyNode.style.top = yPos + "px";
                    
                    var mOver = (function() {
                        var rKey = ii;
                        var cKey = jj;
                        var myX = xPos;
                        var myY = yPos;
                        var hlNode = document.getElementById("Kd_" + getId() + "_KeyHighlighter");

                        var keyLabel = mouseOverKeyLabel;
                        var timesPressedLabel = mouseOverTimesPressedLabel;
                        var avgSpeedLabel = mouseOverAverageSpeedLabel;

                        return function () {
                            hlNode.style.left = (myX-1) + "px";
                            hlNode.style.top = KbKeyHeight * rKey;
                            hlNode.style.width = KbKeyWidths[rKey][cKey] - 2 + "px";
                            hlNode.style.height = KbKeyHeight - 3 + "px";
                            hlNode.style.borderStyle = "solid";
                            hlNode.style.borderWidth = "2px";
                            hlNode.style.display = "block";
                            
                            var kc = "kc-" + KeyCodeMap[ that.getKeyboardLayout() ].layout[rKey][cKey];
                            
                            if (keyLabel) {
                                keyLabel.innerHTML = "[" + KeyCodeLabelMap[ kc ] + "]";
                            }
                            if (timesPressedLabel) {
                                timesPressedLabel.innerHTML = "[" + that.getNumTimesPressed(kc) + "]";
                            }
                            if (avgSpeedLabel) {
                                
                                var avgText = "-";
                                if (that.getNumTimesPressed(kc) > 0) {
                                      avgText = "[" + that.getAvgPressTime(kc).toFixed(0) + " ms]";
                                }
                                
                                avgSpeedLabel.innerHTML =  avgText ;
                            }
                        }
                    })();
                    keyNode.addEventListener("mouseover", mOver, false);

                    var mOut = (function() {
                        
                        var hlNode = document.getElementById("Kd_" + getId() + "_KeyHighlighter");
                        
                        var keyLabel = mouseOverKeyLabel;
                        var timesPressedLabel = mouseOverTimesPressedLabel;
                        var avgSpeedLabel = mouseOverAverageSpeedLabel;
                        
                        return function() {
                            console.log("!!!");

                            hlNode.style.display = "none";
                            
                            if (keyLabel) {
                                keyLabel.innerHTML = "-";
                            }
                            if (timesPressedLabel) {
                                timesPressedLabel.innerHTML = "-";
                            }
                            if (avgSpeedLabel) {
                                avgSpeedLabel.innerHTML = "-";
                            }
                        }
                    })();
                    keyNode.addEventListener("mouseout", mOut, true);
                    
                    dContent.appendChild(keyNode);
                    
                    xPos += KbKeyWidths[ii][jj];
                }
                yPos += KbKeyHeight;
            }
        };
    
        /*
            methods:
                getKeyoutLayoutImg
                getKeyboardLayout
                setKeyboardLayout
            notes:
                Keyboard layout related methods
        */
    
        this.getKeyoutLayoutImg = function() {
            return KeyCodeMap[KbLayout].img;
        };
    
        this.getKeyboardLayout = function() {
            return KbLayout;
        };
    
        this.setKeyboardLayout = function(newLayout) {
            if ( KeyCodeMap[newLayout] ) {
                KbLayout = newLayout;
                
                keyCover = document.getElementById( "Kd_" + getId() + "_KeyCover" );
                if (keyCover) {
                    keyCover.style.backgroundImage = "url('" + this.getKeyoutLayoutImg() + "')";
                }
            }
        };
    
        this.setVisualizationType = function(newType) {
            if ( ValidVisualizationType[newType] ) {
                VisualizationType = newType;
            }
        };
    
        this.getVisualizationType = function() {
            return VisualizationType;
        };
    
        /*
            methods:
                getKeyData
                setKeyData
        */
        this.getKeyData = function() {
            return KeyData;
        };
        
        this.getAvgPressTime = function(kc) {
            if (KeyData[kc]) {
                return KeyData[kc].avg;
            } else {
                return 0;
            }            
        }

        this.getNumTimesPressed = function(kc) {
            if (KeyData[kc]) {
                return KeyData[kc].count;
            } else {
                return 0;
            }
        }
        
        this.setKeyData = function(newData) {
            KeyData = newData;
        };
    
        this.displayVisualization = function() {
              
            console.log("entering...");
        
            var kData = this.getKeyData();
            var keyAvgData = this.getVisualizationLimits();
            if (!keyAvgData.largest) {return;} // no data, let's exit
        
            console.log("about to compute");
        
            var range = keyAvgData.largest - keyAvgData.smallest;
            if (range === 0) {range = 1;} 
            
            console.log("range = " + range);
            
            var layout = this.getKeyboardLayout();
            
            console.log("layout = " + layout);
            console.log("KeyCodeMap[ layout ].layout.length = " + KeyCodeMap[ layout ].layout.length);
            
            // colors for our visualization
            var coolR = 128;
            var coolG = 182;
            var coolB = 229;
            var hotR = 255;
            var hotG = 0;
            var hotB = 0;
            
            // Frequency is backwards from speed, so flip the colors
            if ( this.getVisualizationType() === "frequency") {
                var tmpR = coolR;
                var tmpG = coolG;
                var tmpB = coolB;
                coolR = hotR;
                coolG = hotG;
                coolB = hotB;
                hotR = tmpR;
                hotG = tmpG;
                hotB = tmpB;
            }   
            
            var rr = coolR - hotR;
            var gg = coolG - hotG;
            var bb = coolB - hotB; 
            
            for (var row = 0; row < KeyCodeMap[ layout ].layout.length; row++) {
                for (var col = 0; col < KeyCodeMap[ layout ].layout[row].length; col++) {
                    
                    var kc = "kc-" + KeyCodeMap[ layout ].layout[row][col];
                    
                    if ( kData[ kc ] ) {
                    
                        var myVal;
                        if ( this.getVisualizationType() === "speed" ) {
                            myVal = kData[ kc ].avg;
                        } else if (this.getVisualizationType() === "frequency") {
                            myVal = kData[ kc ].count;
                        }    
                    
                        var percentVal = (myVal - keyAvgData.smallest) / range;
                        var redAmount = hotR + Math.round(rr * percentVal);
                        var greenAmount = hotG + Math.round(gg * percentVal);
                        var blueAmount = hotB + Math.round(bb * percentVal);
                        
                        var d = document.getElementById("Kd_" + getId() + "_KeyHighlighter_" + row + "_" + col);
                        d.style.backgroundColor = "rgb(" + redAmount + "," + greenAmount + "," + blueAmount + ")";
                    }
                }
            }
        };
    
        this.getVisualizationLimits = function() {

            var ret = {};
            var kData = this.getKeyData();
            var layout = this.getKeyboardLayout();

            for (var row = 0; row < KeyCodeMap[ layout ].layout.length; row++) {
                for (var col = 0; col < KeyCodeMap[ layout ].layout[row].length; col++) {
                    
                    var kc = "kc-" + KeyCodeMap[ layout ].layout[row][col];
                    
                    if ( kData[ kc ] ) {
                    
                        var myVal;
                    
                        if ( this.getVisualizationType() === "speed" ) {
                            myVal = kData[ kc ].avg;
                        } else if (this.getVisualizationType() === "frequency") {
                            myVal = kData[ kc ].count;
                        }      
                    
                        if (!ret.smallest || ret.smallest > myVal) {
                            ret.smallest = myVal;
                        }
                        if (!ret.largest || ret.largest < myVal) {
                            ret.largest = myVal;
                        }
                    }
                }
            }
        
            return ret;
        };
    
    	// -------------------------------------------------------------------------
    	// Constructor logic
    	// -------------------------------------------------------------------------
    
        if ( !config ) { 
            config = {};   
        }
    
        if (config.renderTo) {
            this.setup( config );
        }
    };

})();