/*
    File:
        typing-listener.js
    Author:
        Patrick Gillespie - http://patorjk.com/
    Code Licence:
        GPL - http://www.gnu.org/licenses/gpl.html
    notes:
        This code gets injected into each webpage you visit.
*/

var PrevTime = 0;
var KeyData = {};

/*
    function:
        typingSpeedListener
    notes:
        This function will be attached to the document's keyup, keydown or keypress event. 
        It will listen for keyup events and process and save the data in the KeyData array.
*/
function typingSpeedListener(e) {
    var keyNum = "kc-" + e.keyCode;
    console.log("keyNum = " + keyNum);

    var curTime = (new Date()).getTime();
    var typingTime = curTime - PrevTime;
    
    if (typingTime > 0 && typingTime < 5000) {
        
        if (!KeyData[keyNum]) {
            KeyData[keyNum] = {"count":0,"avg":0};
        }
        
        var prevCount = KeyData[keyNum].count;
        KeyData[keyNum].count++;
        KeyData[keyNum].avg = (typingTime + (prevCount * KeyData[keyNum].avg)) / KeyData[keyNum].count;
    }
    
    PrevTime = curTime;
}

/*
    function: 
        storeTypingSpeedData
    notes:
        This routine saves all of the data we have collected to background.html
*/
function storeTypingSpeedData() {
    
    document.removeEventListener("keydown", typingSpeedListener, true);
    
    var requestObj = {};
    requestObj.typingData = KeyData;
    
    chrome.extension.sendRequest( requestObj, function(response) {});
}

/*
    function: 
        chrome.extension.onRequest Listener
    notes:
        This routine listens to messages sent to us by other parts of the extension
*/

chrome.extension.onRequest.addListener(
    function(request, sender, sendResponse) {
    
        if (request.msg == "report statistics") {
        
            // we don't want to record any more information while send and then
            // clear our current collection of statistical data
            document.removeEventListener("keydown", typingSpeedListener, true); 
        
            var requestObj = {};
            requestObj.typingData = KeyData;
        
            chrome.extension.sendRequest( requestObj, function(response) {});
            
            // clear data since it has been reported
            KeyData = {}; 
            
            // start recording again
            document.addEventListener("keydown", typingSpeedListener, true);
            
            sendResponse({});
        } else {
            sendResponse({}); // snub them.
        }
    }
);

document.addEventListener("keydown", typingSpeedListener, true);
window.addEventListener("beforeunload", storeTypingSpeedData, true);